( cd usr/lib ; rm -rf libglibmm-utils.so.2 )
( cd usr/lib ; ln -sf libglibmm-utils.so.2.0.0 libglibmm-utils.so.2 )
( cd usr/lib ; rm -rf libglibmm-utils.so )
( cd usr/lib ; ln -sf libglibmm-utils.so.2.0.0 libglibmm-utils.so )
( cd usr/lib ; rm -rf libgtkmm-utils.so.2 )
( cd usr/lib ; ln -sf libgtkmm-utils.so.2.0.0 libgtkmm-utils.so.2 )
( cd usr/lib ; rm -rf libgtkmm-utils.so )
( cd usr/lib ; ln -sf libgtkmm-utils.so.2.0.0 libgtkmm-utils.so )
